from __future__ import absolute_import

import os
import sys

from mirdata import utils

import json
import pytest

if sys.version_info.major == 3:
    builtin_module_name = 'builtins'
else:
    builtin_module_name = '__builtin__'

DEFAULT_DATA_HOME = os.path.join(os.getenv('HOME', '/tmp'), 'mir_datasets')


@pytest.fixture
def mock_validated(mocker):
    return mocker.patch.object(utils, 'check_validated')


@pytest.fixture
def mock_download(mocker):
    return mocker.patch.object(utils, 'download_from_remote')


@pytest.fixture
def mock_untar(mocker):
    return mocker.patch.object(utils, 'untar')


@pytest.fixture
def mock_unzip(mocker):
    return mocker.patch.object(utils, 'unzip')


@pytest.fixture
def mock_validator(mocker):
    return mocker.patch.object(utils, 'validator')


@pytest.fixture
def mock_force_delete_all(mocker):
    return mocker.patch.object(utils, 'force_delete_all')


@pytest.fixture
def mock_check_index(mocker):
    return mocker.patch.object(utils, 'check_index')


@pytest.fixture
def mock_create_invalid(mocker):
    return mocker.patch.object(utils, 'create_invalid')


@pytest.fixture
def mock_create_validated(mocker):
    return mocker.patch.object(utils, 'create_validated')


def test_md5(mocker):
    audio_file = b'audio1234'

    expected_checksum = '6dc00d1bac757abe4ea83308dde68aab'

    mocker.patch('%s.open' % builtin_module_name, new=mocker.mock_open(read_data=audio_file))

    md5_checksum = utils.md5('test_file_path')
    assert expected_checksum == md5_checksum


@pytest.mark.parametrize('test_index,expected_missing,expected_inv_checksum', [
    ('test_index_valid.json', {}, {}),
    ('test_index_missing_file.json', {'10161_chorus': ['tests/resources/10162_chorus.wav']}, {}),
    ('test_index_invalid_checksum.json', {}, {'10161_chorus': ['tests/resources/10161_chorus.wav']}),
])
def test_check_index(test_index,
                     expected_missing,
                     expected_inv_checksum):
    index_path = os.path.join('tests/indexes', test_index)
    with open(index_path) as index_file:
        test_index = json.load(index_file)

    missing_files, invalid_checksums = utils.check_index(test_index, 'tests/resources/', True)

    assert expected_missing == missing_files
    assert expected_inv_checksum == invalid_checksums


@pytest.mark.parametrize('missing_files,invalid_checksums', [
    ({'10161_chorus': ['tests/resources/10162_chorus.wav']}, {}),
    ({}, {'10161_chorus': ['tests/resources/10161_chorus.wav']}),
])
def test_validator(mocker,
                   mock_check_index,
                   missing_files,
                   invalid_checksums):
    mock_check_index.return_value = missing_files, invalid_checksums

    m, c = utils.validator('foo', 'bar', True)
    assert m == missing_files
    assert c == invalid_checksums
    mock_check_index.assert_called_once_with('foo', 'bar', True)

    # if missing_files or invalid_checksums:
    #     mock_create_invalid.assert_called_once_with(missing_files, invalid_checksums)
    # else:
    #     mock_create_validated.assert_called_once_with('baz')


def test_get_default_dataset_path():
    assert '/tmp/mir_datasets/data_home' == utils.get_default_dataset_path('data_home')


def test_download_from_remote(httpserver, tmpdir):
    httpserver.serve_content(open('tests/resources/remote.wav').read())

    TEST_REMOTE = utils.RemoteFileMetadata(
        filename='remote.wav',
        url=httpserver.url,
        checksum=('3f77d0d69dc41b3696f074ad6bf2852f')
    )

    download_path = utils.download_from_remote(TEST_REMOTE, str(tmpdir))
    expected_download_path = os.path.join(str(tmpdir), 'remote.wav')
    assert expected_download_path == download_path


def test_download_from_remote_raises_IOError(httpserver, tmpdir):
    httpserver.serve_content('File not found!', 404)

    TEST_REMOTE = utils.RemoteFileMetadata(
        filename='remote.wav',
        url=httpserver.url,
        checksum=('1234')
    )

    with pytest.raises(IOError):
        utils.download_from_remote(TEST_REMOTE, str(tmpdir))


def test_unzip(tmpdir):
    utils.unzip('tests/resources/remote.zip', str(tmpdir))

    expected_file_location = os.path.join(str(tmpdir), 'remote.wav')
    assert os.path.exists(expected_file_location)


def test_untar(tmpdir):
    utils.untar('tests/resources/remote.tar.gz', str(tmpdir))

    expected_file_location = os.path.join(str(tmpdir), 'remote.wav')
    assert os.path.exists(expected_file_location)


# def test_check_validated(tmpdir):
#     tmpdir_str = str(tmpdir)
#     assert not utils.check_validated(tmpdir_str)

#     utils.create_validated(tmpdir_str)
#     assert utils.check_validated(tmpdir_str)


# def test_create_validated(tmpdir):
#     tmpdir_str = str(tmpdir)
#     expected_validated_path = os.path.join(tmpdir_str, utils.VALIDATED_FILE_NAME)
#     assert not os.path.exists(expected_validated_path)

#     utils.create_validated(tmpdir_str)
#     assert os.path.exists(expected_validated_path)
#     with open(expected_validated_path, 'r') as f:
#         # Yes we could do not f.read(), but the intentions here are clearer
#         assert f.read() == ''


# @pytest.mark.parametrize('missing_files,invalid_checksums', [
#     ({'10161_chorus': ['tests/resources/10162_chorus.wav']}, {}),
#     ({}, {'10161_chorus': ['tests/resources/10161_chorus.wav']}),
# ])
# def test_create_invalid(tmpdir,
#                         missing_files,
#                         invalid_checksums):
#     tmpdir_str = str(tmpdir)
#     utils.create_invalid(tmpdir_str, missing_files, invalid_checksums)

#     with open(os.path.join(tmpdir_str, utils.INVALID_FILE_NAME)) as f:
#         invalid_content = json.load(f)
#         assert invalid_content == {'missing_files': missing_files,
#                                    'invalid_checksums': invalid_checksums}


def test_force_delete_all_nonempty_data_home(httpserver, tmpdir):
    tmpdir_str = str(tmpdir)
    remote_filename = 'remote.wav'
    TEST_REMOTE = utils.RemoteFileMetadata(
        filename=remote_filename,
        url=httpserver.url,
        checksum=('1234')
    )

    with pytest.raises(IOError):
        utils.download_from_remote(TEST_REMOTE, tmpdir_str)

    utils.untar('tests/resources/remote.tar.gz', tmpdir_str)
    assert os.path.exists(os.path.join(tmpdir_str, remote_filename))
    assert os.path.exists(tmpdir_str)
    utils.force_delete_all(TEST_REMOTE, tmpdir_str)
    assert not os.path.exists(os.path.join(tmpdir_str, remote_filename))
    assert not os.path.exists(tmpdir_str)



# -*- coding: utf-8 -*-
"""Utility functions for mirdata

Attributes:
    MIR_DATASETS_DIR (str): home folder for MIR datasets

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from collections import namedtuple
import hashlib
import os
import json
import tarfile
import shutil
import zipfile
import requests
from requests.exceptions import HTTPError
from tqdm import tqdm

MIR_DATASETS_DIR = os.path.join(os.getenv('HOME', '/tmp'), 'mir_datasets')
VALIDATED_FILE_NAME = '_VALIDATED'
INVALID_FILE_NAME = '_INVALID.json'


def md5(file_path):
    """Get md5 hash of a file.

    Args:
        file_path (str): File path

    Returns:
        md5_hash (str): md5 hash of data in file_path

    """
    hash_md5 = hashlib.md5()
    with open(file_path, 'rb') as fhandle:
        for chunk in iter(lambda: fhandle.read(4096), b''):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def log_message(message, silence=False):
    """Helper function to log message

    Args:
        message (str): message to log
        silence (bool): if true, the message is not logged
    """
    if not silence:
        print(message)


def check_index(dataset_index, data_home, silence=False):
    """check index to find out missing files and files with invalid checksum

    Args:
        dataset_index (list): dataset indices
        data_home (str): Local home path that the dataset is being stored
        silence (bool)

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    missing_files = {}
    invalid_checksums = {}

    # loop over track ids
    for track_id, track in dataset_index.items():
        # loop over each data file for this track id
        for key in track.keys():
            filepath = track[key][0]
            checksum = track[key][1]
            if filepath is not None:
                local_path = os.path.join(data_home, filepath)
                # validate that the file exists on disk
                if not os.path.exists(local_path):
                    if track_id not in missing_files.keys():
                        missing_files[track_id] = []
                    missing_files[track_id].append(local_path)
                # validate that the checksum matches
                elif md5(local_path) != checksum:
                    if track_id not in invalid_checksums.keys():
                        invalid_checksums[track_id] = []
                    invalid_checksums[track_id].append(local_path)

    return missing_files, invalid_checksums


def validator(dataset_index, data_home, silence=False):
    """validate.. (todo: what does it do?) """
    missing_files, invalid_checksums = check_index(dataset_index, data_home, silence)

    # print path of any missing files
    for track_id in missing_files.keys():
        if len(missing_files[track_id]) > 0:
            log_message('Files missing for {}:'.format(track_id), silence)
            for fpath in missing_files[track_id]:
                log_message(fpath, silence)
            log_message('-' * 20, silence)

    # print path of any invalid checksums
    for track_id in invalid_checksums.keys():
        if len(invalid_checksums[track_id]) > 0:
            log_message('Invalid checksums for {}:'.format(track_id), silence)
            for fpath in invalid_checksums[track_id]:
                log_message(fpath, silence)
            log_message('-' * 20, silence)

    return missing_files, invalid_checksums


F0Data = namedtuple('F0Data', ['times', 'frequencies', 'confidence'])

LyricData = namedtuple(
    'LyricData', ['start_times', 'end_times', 'lyrics', 'pronounciations']
)

SectionData = namedtuple('SectionData', ['start_times', 'end_times', 'sections'])

BeatData = namedtuple('BeatData', ['beat_times', 'beat_positions'])

ChordData = namedtuple('ChordData', ['start_times', 'end_times', 'chords'])

KeyData = namedtuple('KeyData', ['start_times', 'end_times', 'keys'])


def get_default_dataset_path(dataset_name):
    """Get the default path for a dataset given it's name

    Args:
        dataset_name (str or None)
            The name of the dataset folder, e.g. 'Orchset'

    Returns:
        save_path (str): Local path to the dataset
    """
    return os.path.join(MIR_DATASETS_DIR, dataset_name)


RemoteFileMetadata = namedtuple('RemoteFileMetadata', ['filename', 'url', 'checksum'])


class DownloadProgressBar(tqdm):
    """Wrap `tqdm` to show download progress"""
    def update_to(self, b=1, bsize=1, tsize=None):
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)


def download_large_file(url, download_path, callback=lambda: None):
    """download large file stably (todo: ??)"""
    response = requests.get(url)
    response.raise_for_status()
    with open(download_path, 'wb') as handle:
        for block in response.iter_content(4096):
            handle.write(block)
            callback()
    return download_path


def download_from_remote(remote, data_home, force_overwrite=False):
    """Download a remote dataset into path
    Fetch a dataset pointed by remote's url, save into path using remote's
    filename and ensure its integrity based on the MD5 Checksum of the
    downloaded file.

    Adapted from scikit-learn's sklearn.datasets.base._fetch_remote.

    Parameters
    -----------
    remote: RemoteFileMetadata
        Named tuple containing remote dataset meta information: url, filename
        and checksum
    data_home: string
        Directory to save the file to.
    force_overwrite: bool
        If True, overwrite existing file with the downloaded file.
        If False, does not overwrite, but checks that checksum is consistent.

    Returns
    -------
    file_path: string
        Full path of the created file.
    """
    download_path = os.path.join(data_home, remote.filename)
    if not os.path.exists(download_path) or force_overwrite:
        # If file doesn't exist or we want to overwrite, download it
        with DownloadProgressBar(
            unit='B', unit_scale=True, miniters=1, desc=remote.url.split('/')[-1]
        ) as t:
            try:
                download_large_file(remote.url, download_path, t.update_to)
            except HTTPError:
                error_msg = """
                            mirdata failed to download the dataset!
                            Please try again in a few minutes.
                            If this error persists, please raise an issue at
                            https://github.com/mir-dataset-loaders/mirdata,
                            and tag it with 'broken-link'.
                            """
                raise HTTPError(error_msg)

    checksum = md5(download_path)
    if remote.checksum != checksum:
        raise IOError(
            '{} has an MD5 checksum ({}) '
            'differing from expected ({}), '
            'file may be corrupted.'.format(download_path, checksum, remote.checksum)
        )
    return download_path


def unzip(zip_path, save_dir, cleanup=False):
    """Unzip a zip file to a specified save location.

    Parameters
    ----------
    zip_path: str
        Path to zip file
    save_dir: str
        Path to save unzipped data
    cleanup: bool, default=False
        If True, remove zipfile after unzipping.
    """
    zfile = zipfile.ZipFile(zip_path, 'r')
    zfile.extractall(save_dir)
    zfile.close()
    if cleanup:
        os.remove(zip_path)


def untar(tar_path, save_dir, cleanup=False):
    """Untar a tar file to a specified save location.

    Parameters
    ----------
    tar_path: str
        Path to tar file
    save_dir: str
        Path to save untarred data
    cleanup: bool, default=False
        If True, remove tarfile after untarring.
    """
    if tar_path.endswith('tar.gz'):
        tfile = tarfile.open(tar_path, 'r:gz')
    else:
        tfile = tarfile.TarFile(tar_path, 'r')
    tfile.extractall(save_dir)
    tfile.close()
    if cleanup:
        os.remove(tar_path)


def load_json_index(filename):
    CWD = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(CWD, 'indexes', filename)) as f:
        return json.load(f)


def force_delete_all(remote, data_home):
    """todo
    """
    if remote:
        download_path = os.path.join(data_home, remote.filename)
        os.remove(download_path)

    shutil.rmtree(data_home)


class cached_property(object):
    """ A property that is only computed once per instance and then replaces
        itself with an ordinary attribute. Deleting the attribute resets the
        property.
        Source: https://github.com/bottlepy/bottle/commit/fa7733e075da0d790d809aa3d2f53071897e6f76
    """

    def __init__(self, func):
        self.__doc__ = getattr(func, "__doc__")
        self.func = func

    def __get__(self, obj, cls):
        # type: (Any, type) -> Any
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value



# -*- coding: utf-8 -*-
"""ORCHSET Dataset Loader

Orchset is intended to be used as a dataset for the development and
evaluation of melody extraction algorithms. This collection contains
64 audio excerpts focused on symphonic music with their corresponding
annotation of the melody.

Details can be found at https://zenodo.org/record/1289786#.XREpzaeZPx6


Attributes:
    INDEX (dict): {track_id: track_data}.
        track_data is a jason data loaded from `index/`

    DIR (str): The directory name for ORCHSET.
        Set to `'Orchset'`.

    METADATA

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
import csv
import librosa
import numpy as np
import os
try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # python 2 backport

import mirdata.utils as utils

INDEX = utils.load_json_index('orchset_index.json')

REMOTE = utils.RemoteFileMetadata(
    filename='Orchset_dataset_0.zip',
    url='https://zenodo.org/record/1289786/files/Orchset_dataset_0.zip?download=1',
    checksum=('cf6fe52d64624f61ee116c752fb318ca'),
)

DATASET_DIR = 'Orchset'
METADATA = None


class Track(object):
    """ORCHSET Track class

    Args:
        track_id (str): track id of the Track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path_mono (str): mono audio path of the track
        audio_path_stereo (str): stereo audio path of the track
        composer (str): composer
        work (str):
        predominant_melodic_instruments (str):
        alternating_melody
        contains_winds (bool?)
        contains_strings
        contains_brass
        only_strings
        only_winds
        only_brass
        melody (F0Data): melody annotation

    """
    def __init__(self, track_id, data_home=None):
        if track_id not in INDEX:
            raise ValueError('{} is not a valid track ID in Orchset'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = INDEX[track_id]

        if METADATA is None or METADATA['data_home'] != data_home:
            _reload_metadata(data_home)

        if METADATA is not None and track_id in METADATA:
            self._track_metadata = METADATA[track_id]
        else:
            self._track_metadata = {
                'predominant_melodic_instruments-raw': None,
                'predominant_melodic_instruments-normalized': None,
                'alternating_melody': None,
                'contains_winds': None,
                'contains_strings': None,
                'contains_brass': None,
                'only_strings': None,
                'only_winds': None,
                'only_brass': None,
                'composer': None,
                'work': None,
                'excerpt': None,
            }

        self.audio_path_mono = os.path.join(
            self._data_home, self._track_paths['audio_mono'][0])
        self.audio_path_stereo = os.path.join(
            self._data_home, self._track_paths['audio_stereo'][0])
        self.composer = self._track_metadata['composer']
        self.work = self._track_metadata['work']
        self.excerpt = self._track_metadata['excerpt']
        self.predominant_melodic_instruments = \
            self._track_metadata['predominant_melodic_instruments-normalized']
        self.alternating_melody = self._track_metadata['alternating_melody']
        self.contains_winds = self._track_metadata['contains_winds']
        self.contains_strings = self._track_metadata['contains_strings']
        self.contains_brass = self._track_metadata['contains_brass']
        self.only_strings = self._track_metadata['only_strings']
        self.only_winds = self._track_metadata['only_winds']
        self.only_brass = self._track_metadata['only_brass']

    def __repr__(self):
        repr_string = "Orchset Track(track_id={}, audio_path_stereo={}, " + \
            "audio_path_mono={}, composer={}, work={}, excerpt={}, " + \
            "predominant_melodic_instruments={}, alternating_melody={}, " + \
            "contains_winds={}, contains_strings={}, contains_brass={}, " + \
            "only_strings={}, only_winds={}, only_brass={}, " + \
            "melody=F0Data('times', 'frequencies', 'confidence'))"
        return repr_string.format(
            self.track_id,
            self.audio_path_stereo,
            self.audio_path_mono,
            self.composer,
            self.work,
            self.excerpt,
            self.predominant_melodic_instruments,
            self.alternating_melody,
            self.contains_winds,
            self.contains_strings,
            self.contains_brass,
            self.only_strings,
            self.only_winds,
            self.only_brass,
        )

    @utils.cached_property
    def melody(self):
        return _load_melody(os.path.join(
            self._data_home, self._track_paths['melody'][0]))

    @property
    def audio_mono(self):
        return librosa.load(self.audio_path_mono, sr=None)

    @property
    def audio_stereo(self):
        return librosa.load(self.audio_path_stereo, sr=None, mono=False)


def download(data_home=None, force_overwrite=False):
    """Download ORCHSET Dataset.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool): whether to overwrite the existing downloaded data

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    if os.path.exists(data_home) and not force_overwrite:
        return

    if force_overwrite:
        utils.force_delete_all(REMOTE, data_home)

    Path(data_home).mkdir(exist_ok=True)

    download_path = utils.download_from_remote(
        REMOTE, force_overwrite=force_overwrite
    )
    utils.unzip(download_path, data_home)


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        dataset_path (str): ORCHSET dataset local path
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """

    missing_files, invalid_checksums = utils.validator(
        INDEX, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(INDEX.keys())


def load(data_home=None, silence_validator=False):
    """Load ORCHSET dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """

    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    validate(data_home, silence=silence_validator)
    orchset_data = {}
    for key in track_ids():
        orchset_data[key] = Track(key, data_home=data_home)
    return orchset_data


def _load_melody(melody_path):
    if not os.path.exists(melody_path):
        return None

    times = []
    freqs = []
    confidence = []
    with open(melody_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            times.append(float(line[0]))
            freqs.append(float(line[1]))
            confidence.append(0.0 if line[1] == '0' else 1.0)

    melody_data = utils.F0Data(np.array(times), np.array(freqs), np.array(confidence))
    return melody_data


def _load_metadata(data_home):

    predominant_inst_path = os.path.join(
        data_home,
        'Orchset - Predominant Melodic Instruments.csv',
    )

    if not os.path.exists(predominant_inst_path):
        print("Warning: metadata file {} not found.".format(predominant_inst_path))
        return None

    with open(predominant_inst_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter=',')
        raw_data = []
        for line in reader:
            if line[0] == 'excerpt':
                continue
            raw_data.append(line)

    tf_dict = {'TRUE': True, 'FALSE': False}

    metadata_index = {}
    for line in raw_data:
        track_id = line[0].split('.')[0]

        id_split = track_id.split('.')[0].split('-')
        if id_split[0] == 'Musorgski' or id_split[0] == 'Rimski':
            id_split[0] = '-'.join(id_split[:2])
            id_split.pop(1)

        melodic_instruments = [s.split(',') for s in line[1].split('+')]
        melodic_instruments = [
            item.lower() for sublist in melodic_instruments for item in sublist
        ]
        for i, inst in enumerate(melodic_instruments):
            if inst == 'string':
                melodic_instruments[i] = 'strings'
            elif inst == 'winds (solo)':
                melodic_instruments[i] = 'winds'
        melodic_instruments = sorted(list(set(melodic_instruments)))

        metadata_index[track_id] = {
            'predominant_melodic_instruments-raw': line[1],
            'predominant_melodic_instruments-normalized': melodic_instruments,
            'alternating_melody': tf_dict[line[2]],
            'contains_winds': tf_dict[line[3]],
            'contains_strings': tf_dict[line[4]],
            'contains_brass': tf_dict[line[5]],
            'only_strings': tf_dict[line[6]],
            'only_winds': tf_dict[line[7]],
            'only_brass': tf_dict[line[8]],
            'composer': id_split[0],
            'work': '-'.join(id_split[1:-1]),
            'excerpt': id_split[-1][2:],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


def _reload_metadata(data_home):
    global METADATA
    METADATA = _load_metadata(data_home=data_home)


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========
Bosch, J., Marxer, R., Gomez, E., "Evaluation and Combination of
Pitch Estimation Methods for Melody Extraction in Symphonic
Classical Music", Journal of New Music Research (2016)

========== Bibtex ==========
@article{bosch2016evaluation,
    title={Evaluation and combination of pitch estimation methods for melody extraction in symphonic classical music},
    author={Bosch, Juan J and Marxer, Ricard and G{\'o}mez, Emilia},
    journal={Journal of New Music Research},
    volume={45},
    number={2},
    pages={101--117},
    year={2016},
    publisher={Taylor \\& Francis}
"""

    print(cite_data)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from . import medleydb_melody
from . import medleydb_pitch
from . import orchset
from . import ikala
from . import beatles
from . import salami



from __future__ import absolute_import

import numpy as np
import os
import pytest
from mirdata import salami, utils
from tests.test_utils import (mock_download, mock_unzip,
                              mock_validator, mock_force_delete_all,
                              DEFAULT_DATA_HOME)


def test_track():
    # test data home None
    track_default = salami.Track('2')
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, 'Salami')

    # test specific data home
    data_home = 'tests/resources/mir_datasets/Salami'

    with pytest.raises(ValueError):
        salami.Track('asdfasdf', data_home=data_home)

    track = salami.Track('2', data_home=data_home)

    # test attributes are loaded as expected
    assert track.track_id == '2'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': [
            'audio/2.mp3',
            '76789a17bda0dd4d1d7e77424099c814'
        ],
        'annotator_1_uppercase': [
            'salami-data-public-master/annotations/2/parsed/textfile1_uppercase.txt',
            '54ba0804f720d85d195dcd7ffaec0794'
        ],
        'annotator_1_lowercase': [
            'salami-data-public-master/annotations/2/parsed/textfile1_lowercase.txt',
            '30ff127ff68c61039b94a44ab6ddda34'
        ],
        'annotator_2_uppercase': [
            'salami-data-public-master/annotations/2/parsed/textfile2_uppercase.txt',
            'e9dca8577f028d3505ff1e5801397b2f'
        ],
        'annotator_2_lowercase': [
            'salami-data-public-master/annotations/2/parsed/textfile2_lowercase.txt',
            '546a783c7b8bf96f2d718c7a4f114699'
        ]
    }
    assert track.audio_path == 'tests/resources/mir_datasets/Salami/' + \
        'audio/2.mp3'

    assert track.source == 'Codaich'
    assert track.annotator_1_id == '5'
    assert track.annotator_2_id == '8'
    assert track.duration_sec == '264'
    assert track.title == 'For_God_And_Country'
    assert track.artist == 'The_Smashing_Pumpkins'
    assert track.annotator_1_time == '37'
    assert track.annotator_2_time == '45'
    assert track.broad_genre == 'popular'
    assert track.genre == 'Alternative_Pop___Rock'

    # test that cached properties don't fail and have the expected type
    assert type(track.sections_annotator_1_uppercase) is utils.SectionData
    assert type(track.sections_annotator_1_lowercase) is utils.SectionData
    assert type(track.sections_annotator_2_uppercase) is utils.SectionData
    assert type(track.sections_annotator_2_lowercase) is utils.SectionData

    # # test audio loading functions
    # y, sr = track.audio
    # assert sr == 44100
    # assert y.shape == (89856, )

    repr_string = "Salami Track(track_id=2, " + \
        "audio_path=tests/resources/mir_datasets/Salami/audio/2.mp3, " + \
        "source=Codaich, title=For_God_And_Country, " + \
        "artist=The_Smashing_Pumpkins, duration_sec=264, annotator_1_id=5, " + \
        "annotator_2_id=8, annotator_1_time=37, annotator_2_time=45, " + \
        "broad_genre=popular, genre=Alternative_Pop___Rock, " + \
        "sections_annotator_1_uppercase=SectionData('start_times', 'end_times', 'sections'), " + \
        "sections_annotator_1_lowercase=SectionData('start_times', 'end_times', 'sections'), " + \
        "sections_annotator_2_uppercase=SectionData('start_times', 'end_times', 'sections'), " + \
        "sections_annotator_2_lowercase=SectionData('start_times', 'end_times', 'sections')"
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = salami.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 1359


def test_load():
    data_home = 'tests/resources/mir_datasets/Salami'
    salami_data = salami.load(data_home=data_home, silence_validator=True)
    assert type(salami_data) is dict
    assert len(salami_data.keys()) == 1359

    # data home default
    salami_data_default = salami.load(silence_validator=True)
    assert type(salami_data_default) is dict
    assert len(salami_data_default.keys()) == 1359


def test_load_sections():
    # load a file which exists
    sections_path = 'tests/resources/mir_datasets/Salami/' + \
        'salami-data-public-master/annotations/2/parsed/textfile1_uppercase.txt'
    section_data = salami._load_sections(sections_path)

    # check types
    assert type(section_data) == utils.SectionData
    assert type(section_data.start_times) is np.ndarray
    assert type(section_data.end_times) is np.ndarray
    assert type(section_data.sections) is np.ndarray

    # check valuess
    assert np.array_equal(section_data.start_times, np.array(
        [0.0, 0.464399092, 14.379863945, 263.205419501]))
    assert np.array_equal(section_data.end_times, np.array(
        [0.464399092, 14.379863945, 263.205419501, 264.885215419]))
    assert np.array_equal(section_data.sections, np.array(
        ['Silence', 'A', 'B', 'Silence']))

    # load a file which doesn't exist
    section_data_none = salami._load_sections('fake/file/path')
    assert section_data_none is None

    # load none
    section_data_none2 = salami._load_sections('asdf/asdf')
    assert section_data_none2 is None


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/Salami'
    metadata = salami._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['2'] == {
        'source': 'Codaich',
        'annotator_1_id': '5',
        'annotator_2_id': '8',
        'duration_sec': '264',
        'title': 'For_God_And_Country',
        'artist': 'The_Smashing_Pumpkins',
        'annotator_1_time': '37',
        'annotator_2_time': '45',
        'class': 'popular',
        'genre': 'Alternative_Pop___Rock',
    }

    none_metadata = salami._load_metadata('asdf/asdf')
    assert none_metadata is None


def test_cite():
    salami.cite()


@pytest.fixture
def mock_validate(mocker):
    return mocker.patch.object(salami, 'validate')


@pytest.fixture
def mock_load_sections(mocker):
    return mocker.patch.object(salami, '_load_sections')


@pytest.fixture
def data_home(tmpdir):
    return str(tmpdir)


@pytest.fixture
def mock_salami_exists(mocker):
    return mocker.patch.object(os.path, 'exists')


def test_download_already_exists(data_home, mocker,
                                 mock_force_delete_all,
                                 mock_salami_exists,
                                 mock_validator,
                                 mock_download,
                                 mock_unzip):
    mock_salami_exists.return_value = True

    salami.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_salami_exists.assert_called_once()
    mock_download.assert_not_called()
    mock_unzip.assert_not_called()
    mock_validator.assert_not_called()


def test_download_clean(data_home,
                        mocker,
                        mock_force_delete_all,
                        mock_salami_exists,
                        mock_download,
                        mock_unzip,
                        mock_validate):

    mock_salami_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''
    mock_validate.return_value = (False, False)

    salami.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_salami_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_download_force_overwrite(data_home,
                          mocker,
                          mock_force_delete_all,
                          mock_salami_exists,
                          mock_download,
                          mock_unzip,
                          mock_validate):

    mock_salami_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''
    mock_validate.return_value = (False, False)

    salami.download(data_home, force_overwrite=True)

    mock_force_delete_all.assert_called_once_with(salami.ANNOTATIONS_REMOTE, data_home=data_home)
    mock_salami_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_validate_invalid(data_home, mocker, mock_validator):
    mock_validator.return_value = (True, True)

    missing_files, invalid_checksums = salami.validate(data_home)
    assert missing_files and invalid_checksums
    mock_validator.assert_called_once()


def test_validate_valid(data_home, mocker, mock_validator):
    mock_validator.return_value = (False, False)

    missing_files, invalid_checksums = salami.validate(data_home)
    assert not (missing_files or invalid_checksums)
    mock_validator.assert_called_once()


def test_load_track_invalid_track_id():
    with pytest.raises(ValueError):
        salami.Track('a_fake_track')


def test_track_ids():
    assert salami.track_ids() == list(salami.INDEX.keys())
